#include <mqx.h>
#include <bsp.h>
#include <io_gpio.h>
#include "pasek.h"
#include "math.h"

/* Task IDs */
#define MAIN_TASK 5
#define HMC5883_ADDRESS				0x1E


LWSEM_STRUCT lock;

/* Function prototypes */
extern void main_task(uint_32);

char fract[] = {'0','1','1','2','3','3','4','4','5','6','6','7','8','8','9','9'};
char piksel;
short i;
unsigned int czas;


const TASK_TEMPLATE_STRUCT  MQX_template_list[] = 
{
  /* Task Index,   Function,  Stack,  Priority, Name,      Attributes,          Param, Time Slice */ 
   { MAIN_TASK,    main_task, 1500,   8,        "service", MQX_AUTO_START_TASK, 0,     0 },
   { 0 }
};

int ReadHMC5883Reg(MQX_FILE_PTR f, uint_8 RegAddr, char * Buffer, unsigned char BytesToRead)
{
		_lwsem_wait(&lock);
		
		if (fwrite(&RegAddr, 1, 1, f) != 1)
		{
				_lwsem_post(&lock);
				return 0;
		}
		
		if (fflush(f) != MQX_OK)
		{
				_lwsem_post(&lock);
				return 0;
		} 
		
		if (ioctl(f, IO_IOCTL_I2C_REPEATED_START, NULL) != I2C_OK)
		{
				_lwsem_post(&lock);
				return 0;
		} 
		
		if (ioctl(f, IO_IOCTL_I2C_SET_RX_REQUEST, &BytesToRead) != I2C_OK)
		{
				_lwsem_post(&lock);
				return 0;
		} 
		
		if (fread(Buffer, 1, BytesToRead, f) != BytesToRead)
		{
				_lwsem_post(&lock);
				return 0;
		} 
		
		if (ioctl(f, IO_IOCTL_I2C_STOP, NULL) != I2C_OK)
		{
				_lwsem_post(&lock);
				return 0;
		}
		
		_lwsem_post (&lock);
		
		return 1;
}		

int WriteHMC5883Reg(MQX_FILE_PTR f, uint_8 RegAddr, char Data)
{
		char buf[2];
		_lwsem_wait (&lock);

		buf[0]=RegAddr;
    buf[1]=Data;
	
	
		
    if (fwrite(buf, 1, 2, f) != 2)
    {
				_lwsem_post (&lock);	
				return 0;
    }
	
		if (fflush(f) != MQX_OK)
    {
        _lwsem_post (&lock);	
				return 0;
    }
			
    if (I2C_OK != ioctl (f, IO_IOCTL_I2C_STOP, NULL))
    {
        _lwsem_post (&lock);	
				return 0;
    }
		
    _lwsem_post (&lock);	
		
		return 1;
}


int InitHMC5883(MQX_FILE_PTR * f, _mqx_int Freq)
{
		char data, HMC5883Addr = HMC5883_ADDRESS;
		
		_mqx_int res, fr=100000;
		
		*f = fopen ("i2c1:", NULL);
		
		if (*f == NULL) _task_block ();
	
		if (I2C_OK != ioctl (*f, IO_IOCTL_I2C_SET_BAUD, &fr))
		{
				fclose(*f);
				return 0;
		} 
		
		if (I2C_OK != ioctl (*f, IO_IOCTL_I2C_SET_MASTER_MODE, NULL))
		{
				fclose(*f);
				return 0;
		}
   
		if (I2C_OK != ioctl (*f, IO_IOCTL_I2C_SET_DESTINATION_ADDRESS, &HMC5883Addr))
		{
				fclose(*f);
				return 0;	
		} 
		
		
		WriteHMC5883Reg(*f, 0x01, 0x00); //0.88gaus
		WriteHMC5883Reg(*f, 0x02, 0x00);//Cykliczny pomiar

    return 1;
}

float HMC5883Get(MQX_FILE_PTR f, signed short *x,signed short *y,signed short *z )
{
		char Bufor[6];
		ReadHMC5883Reg(f, 0x03, Bufor, 6);

		*x=(signed short)((Bufor[0]<<8)|Bufor[1]);
		*z=(signed short)((Bufor[2]<<8)|Bufor[3]);
		*y=(signed short)((Bufor[4]<<8)|Bufor[5]);
		return 0;
}


void SLCD_Horyzont(unsigned char *tablica, unsigned short pozycja)
{
	unsigned char i, odnowa=0, poczatek=0;
	float mnoznik = 0.355555;
	unsigned short polozenie;
	polozenie=(unsigned short) (pozycja * mnoznik);
	for (i=0;i<37;i++)
	{
		//jeli koniec tablicy
		if (tablica[i+polozenie]==0x00) poczatek=1;
		if(!poczatek)
		{
			for(piksel=0;piksel<8;piksel++)
			{
				if((tablica[i+polozenie]&(1<<piksel))!=0)
				{
					_SLCDModule_SetSegment(i, piksel);
				}
			}	
		}
		else
		{					
			//Jeli koniec tablicy rysuj obraz od pocztku na dalszej czci lcd	
			for(piksel=0;piksel<8;piksel++)
			{		
				if((tablica[odnowa]&(1<<piksel))!=0)
				{	
					_SLCDModule_SetSegment(i, piksel);
				}
			}
			odnowa++;
		}
	}
}




void main_task 
   (
      uint_32 initial_data
   )
{
	volatile unsigned long int petla =15000; //Czas kalibracji 15s
	signed short x, y,z;
	signed short xmin=32767 ,xmax=-32768,ymin=32767,ymax=-32768;
	float offsetx,offsety, scaledx, scaledy, heading;	
	unsigned short line, bar;
	
	float declinationAngle = 0.0880;
	float headingDegrees=0;

		MQX_FILE_PTR HMC5883File;
		ADC_RESULT_STRUCT data;
	
		_SLCDModule_Init();
		_SLCDModule_ClearLCD(0);
	
	
		if (! InitHMC5883(&HMC5883File, 100000)) 
					_task_block();
		
	LCD_WF3TO0   = 0x00FF1100;
  LCD_WF7TO4   = 0x00000000;
  LCD_WF11TO8  = 0x00000000;
  LCD_WF15TO12 = 0x00000000;
  LCD_WF19TO16 = 0x00000000;
  LCD_WF23TO20 = 0x00000000;
  LCD_WF27TO24 = 0x00000000;
  LCD_WF31TO28 = 0x00000000;
  LCD_WF35TO32 = 0x00000000;
  LCD_WF39TO36 = 0x0000FF00;
		
		_SLCDModule_PrintString("KAL", 11);
		
		while (petla)
	{
		HMC5883Get(HMC5883File, &x,&y,&z);
		
		if (x<xmin) xmin=x;
		if (x>xmax) xmax=x;
		
		if (y<ymin) ymin=y;
		if (y>ymax) ymax=y; 
		
		petla--;
		
		//37
		bar=(unsigned short)petla/405;
		for (line=0;line<8;line++)
		{
		_SLCDModule_SetSegment(bar, line);
		}
	}
	
	//Przesuniecie ukladu wspolrzednych tak aby
	//jego srodek znajdowal sie w polowie roznicy minimalnego i maksymalnego
	//pola magnetycznego odczytanego przez magnetometr dla osi X oraz Y
	offsetx=((xmin+xmax)*0.5);
	if (offsetx<0) offsetx=(-1*offsetx);
	if (xmin<0) xmin=(-1*xmin);
	if ((xmin)<xmax) offsetx=-offsetx;
	
	offsety=((ymin+ymax)*0.5);
	if (offsety<0) offsety=(-1*offsety);
	if (ymin<0) ymin=(-1*ymin);
	if ((ymin)<ymax) offsety=-offsety;
		
		
		while(1)
		{
	
		volatile unsigned long int i;
			
		//Pobranie danych pola magnetycznego
		HMC5883Get(HMC5883File, &x,&y,&z);
		
		//Obliczenie kierunku geograficznego
		//Skalowanie wartosci z czujnika i przesuniecie 
		//ukladu wspolrzednych
		scaledx=(x+offsetx)*0.73;
		scaledy=(y+offsety)*0.73;
		
		//Obliczenie kierunku geograficznego
		//heading = atan2(y, x);
		heading = atan2(scaledy, scaledx);
		heading += declinationAngle+3.1415926;

		if(heading < 0)
    heading += 6.2831853;   //2 pi
  
    if(heading > 6.2831853)
    heading -= 6.2831853;
		//Zamiana wyniku na stopnie
    headingDegrees = (heading * 180/3.1415926); 
		
			

				
				SLCD_Horyzont(pasek,(unsigned short)headingDegrees);
				for(czas=0;czas<200000;czas++);
				
	LCD_WF3TO0   = 0xFFFF1100;
  LCD_WF7TO4   = 0x00000000;
  LCD_WF11TO8  = 0x00000000;
  LCD_WF15TO12 = 0x00000000;
  LCD_WF19TO16 = 0x00000000;
  LCD_WF23TO20 = 0x000000FF;
  LCD_WF27TO24 = 0x00000000;
  LCD_WF31TO28 = 0x00000000;
  LCD_WF35TO32 = 0x00000000;
  LCD_WF39TO36 = 0x00FFFF00;
			

				
		}
}  
